@@ -368,7 +368,7 @@ static int do_cd(char *newdir)
 	all_string_sub(cur_dir, "/./", "/", 0);
 	
 	/* Format the directory in a libmsmbclient friendly way */
-	unix_clean_name(cur_dir);
+	clean_name(cur_dir);
 	all_string_sub(cur_dir, "/./", "/", 0);
 	pstrcpy(targetpath, "smb:");
 	pstrcat(targetpath, service);
@@ -445,7 +445,7 @@ static void display_finfo(file_info *finfo)
 			 finfo->name,
 			 attrib_string(finfo->mode),
 			 (double)finfo->size,
-			 asctime(localtime(&t)));
+			 time_to_asc(t));
 		dir_total += finfo->size;
 	}
 }
@@ -458,7 +458,7 @@ static void display_stat(char *name, struct stat *st)
 {
 	time_t t = st->st_mtime;
 	pstring time_str;
-	pstrcpy(time_str, asctime(localtime(&t)));
+	pstrcpy(time_str, time_to_asc(t));
 	time_str[strlen(time_str)-1] = 0;
 	d_printf("> %-30s", name);
 	d_printf("%10.10s %8.0f  %s\n", *mode_t_string(st->st_mode), (double)st->st_size, time_str);
@@ -528,7 +528,14 @@ static void adjust_do_list_queue(void)
 	 * If the starting point of the queue is more than half way through,
 	 * move everything toward the beginning.
 	 */
-	if (do_list_queue && (do_list_queue_start == do_list_queue_end)) {
+
+	if (do_list_queue == NULL) {
+		DEBUG(4,("do_list_queue is empty\n"));
+		do_list_queue_start = do_list_queue_end = 0;
+		return;
+	}
+		
+	if (do_list_queue_start == do_list_queue_end) {
 		DEBUG(4,("do_list_queue is empty\n"));
 		do_list_queue_start = do_list_queue_end = 0;
 		*do_list_queue = '\0';
@@ -544,19 +551,17 @@ static void adjust_do_list_queue(void)
 
 static void add_to_do_list_queue(const char* entry)
 {
-	char *dlq;
 	long new_end = do_list_queue_end + ((long)strlen(entry)) + 1;
 	while (new_end > do_list_queue_size) {
 		do_list_queue_size *= 2;
 		DEBUG(4,("enlarging do_list_queue to %d\n",
 			 (int)do_list_queue_size));
-		dlq = SMB_REALLOC(do_list_queue, do_list_queue_size);
-		if (! dlq) {
+		do_list_queue = SMB_REALLOC(do_list_queue, do_list_queue_size);
+		if (!do_list_queue) {
 			d_printf("failure enlarging do_list_queue to %d bytes\n",
 				 (int)do_list_queue_size);
 			reset_do_list_queue();
 		} else {
-			do_list_queue = dlq;
 			memset(do_list_queue + do_list_queue_size / 2,
 				   0, do_list_queue_size / 2);
 		}
@@ -1125,7 +1130,7 @@ static int cmd_more(void)
 		unlink(lname);
 		return 1;
 	}
-	dos_clean_name(rname);
+	clean_name(rname);
 
 	rc = do_get(rname, lname, False);
 
@@ -2305,6 +2310,7 @@ static int cmd_stat(void)
 	fstring mode_str;
 	SMB_STRUCT_STAT sbuf;
 	struct cli_state *targetcli;
+	struct tm *lt;
 	pstring targetname;
  
 	if (!SERVER_HAS_UNIX_CIFS(cli)) {
@@ -2359,15 +2365,30 @@ static int cmd_stat(void)
 		(unsigned int)sbuf.st_uid, 
 		(unsigned int)sbuf.st_gid);
 
-	strftime(mode_str, sizeof(mode_str), "%F %T %z", localtime(&sbuf.st_atime));
+	lt = localtime(&sbuf.st_atime);
+	if (lt) {
+		strftime(mode_str, sizeof(mode_str), "%F %T %z", lt);
+	} else {
+		fstrcpy(mode_str, "unknown");
+	}
 	d_printf("Access: %s\n", mode_str);
 
-	strftime(mode_str, sizeof(mode_str), "%F %T %z", localtime(&sbuf.st_mtime));
+	lt = localtime(&sbuf.st_mtime);
+	if (lt) {
+		strftime(mode_str, sizeof(mode_str), "%F %T %z", lt);
+	} else {
+		fstrcpy(mode_str, "unknown");
+	}
 	d_printf("Modify: %s\n", mode_str);
 
-	strftime(mode_str, sizeof(mode_str), "%F %T %z", localtime(&sbuf.st_ctime));
+	lt = localtime(&sbuf.st_ctime);
+	if (lt) {
+		strftime(mode_str, sizeof(mode_str), "%F %T %z", lt);
+	} else {
+		fstrcpy(mode_str, "unknown");
+	}
 	d_printf("Change: %s\n", mode_str);
-	
+
 	return 0;
 }
 
@@ -2540,7 +2561,7 @@ static int cmd_newer(void)
 	if (ok && (sys_stat(buf,&sbuf) == 0)) {
 		newer_than = sbuf.st_mtime;
 		DEBUG(1,("Getting files newer than %s",
-			 asctime(localtime(&newer_than))));
+			 time_to_asc(newer_than)));
 	} else {
 		newer_than = 0;
 	}
@@ -2657,7 +2678,7 @@ static int cmd_reget(void)
 		return 1;
 	}
 	pstrcpy(local_name, p);
-	dos_clean_name(remote_name);
+	clean_name(remote_name);
 	
 	next_token_nr(NULL, local_name, NULL, sizeof(local_name));
 	
@@ -2695,7 +2716,7 @@ static int cmd_reput(void)
 	else
 		pstrcat(remote_name, local_name);
 	
-	dos_clean_name(remote_name);
+	clean_name(remote_name);
 
 	return do_put(remote_name, local_name, True);
 }
@@ -3473,7 +3494,7 @@ static int do_message_op(void)
 
 	msg_port = port ? port : 139;
 
-	if (!(cli=cli_initialise(NULL)) || (cli_set_port(cli, msg_port) != msg_port) ||
+	if (!(cli=cli_initialise()) || (cli_set_port(cli, msg_port) != msg_port) ||
 		!cli_connect(cli, server_name, &ip)) {
 		d_printf("Connection to %s failed\n", desthost);
 		return 1;
@@ -3547,10 +3568,11 @@ static int do_message_op(void)
 	set_global_myname( "" );
 
 		/* set default debug level to 0 regardless of what smb.conf sets */
-	setup_logging( "smbclient", True );
+	setup_logging( "smbctool", True );
 	DEBUGLEVEL_CLASS[DBGC_ALL] = 1;
-	dbf = x_stderr;
-	x_setbuf( x_stderr, NULL );
+	if ((dbf = x_fdup(x_stderr))) {
+		x_setbuf( dbf, NULL );
+	}
 
 	pc = poptGetContext("smbclient", argc, (const char **) argv, long_options, 
 				POPT_CONTEXT_KEEP_FIRST);
@@ -3583,6 +3605,9 @@ static int do_message_op(void)
 			}
 			break;
 		case 'E':
+			if (dbf) {
+				x_fclose(dbf);
+			}
 			dbf = x_stderr;
 			display_set_stderr();
 			break;
@@ -3651,8 +3676,10 @@ static int do_message_op(void)
 	
 	if ( override_logfile )
 		setup_logging( lp_logfile(), False );
-	
-	if (!lp_load(dyn_CONFIGFILE,True,False,False)) {
+
+	load_case_tables();
+
+	if (!lp_load(dyn_CONFIGFILE,True,False,False,True)) {
 		fprintf(stderr, "%s: Can't load %s - run testparm to debug it\n",
 			argv[0], dyn_CONFIGFILE);
 	}
