@@ -140,8 +140,8 @@ struct dcinfo {
 	DOM_CHAL clnt_chal; /* Client credential */
 	DOM_CHAL srv_chal;  /* Server credential */
  
-	uchar  sess_key[8]; /* Session key */
-	uchar  mach_pw[16];   /* md4(machine password) */
+	unsigned char  sess_key[16]; /* Session key - 8 bytes followed by 8 zero bytes */
+	unsigned char  mach_pw[16];   /* md4(machine password) */
 
 	fstring mach_acct;  /* Machine name we've authenticated. */
 
@@ -187,7 +187,7 @@ struct kerberos_auth_struct {
 
 /* auth state for schannel. */
 struct schannel_auth_struct {
-	uchar sess_key[16];
+	unsigned char sess_key[16];
 	uint32 seq_num;
 };
 
@@ -266,6 +266,12 @@ typedef struct pipes_struct {
 
 	BOOL bad_handle_fault_state;
 	
+	/*
+	 * Set to true when the backend does not support a call.
+	 */
+
+	BOOL rng_fault_state;
+	
 	/*
 	 * Set to RPC_BIG_ENDIAN when dealing with big-endian PDU's
 	 */
@@ -284,7 +290,7 @@ typedef struct pipes_struct {
 
 	output_data out_data;
 
-	/* This context is used for PUD data and is freed between each pdu.
+	/* This context is used for PDU data and is freed between each pdu.
 		Don't use for pipe state storage. */
 	TALLOC_CTX *mem_ctx;
 
