@@ -252,12 +252,6 @@ NTSTATUS idmap_close(void)
 	return NT_STATUS_OK;
 }
 
-/**********************************************************************
- Initialise idmap cache and a remote backend (if configured).
-**********************************************************************/
-
-static const char *idmap_default_domain[] = { "default domain", NULL };
-
 /****************************************************************************
  ****************************************************************************/
 
@@ -292,6 +286,7 @@ NTSTATUS idmap_init(void)
 	char *compat_backend = NULL;
 	char *compat_params = NULL;
 	const char **dom_list = NULL;
+	const char *default_domain = NULL;
 	char *alloc_backend = NULL;
 	BOOL default_already_defined = False;
 	BOOL pri_dom_is_in_list = False;
@@ -321,12 +316,6 @@ NTSTATUS idmap_init(void)
 		} else {
 			compat = 1;
 
-			compat_backend = talloc_strdup(idmap_ctx, *compat_list);
-			if (compat_backend == NULL) {
-				ret = NT_STATUS_NO_MEMORY;
-				goto done;
-			}
-
 			/* strip any leading idmap_ prefix of */
 			if (strncmp(*compat_list, "idmap_", 6) == 0 ) {
 				q = *compat_list += 6;
@@ -340,6 +329,11 @@ NTSTATUS idmap_init(void)
 								*compat_list);
 			}
 
+			if (compat_backend == NULL) {
+				ret = NT_STATUS_NO_MEMORY;
+				goto done;
+			}
+
 			/* separate the backend and module arguements */
 			if ((p = strchr(compat_backend, ':')) != NULL) {
 				*p = '\0';
@@ -356,7 +350,25 @@ NTSTATUS idmap_init(void)
 	}
 
 	if ( ! dom_list) {
-		dom_list = idmap_default_domain;
+		/* generate a list with our main domain */
+		char ** dl;
+
+		dl = talloc_array(idmap_ctx, char *, 2);
+		if (dl == NULL) {
+			ret = NT_STATUS_NO_MEMORY;
+			goto done;
+		}
+		dl[0] = talloc_strdup(dl, lp_workgroup());
+		if (dl[0] == NULL) {
+			ret = NT_STATUS_NO_MEMORY;
+			goto done;
+		}
+
+		/* terminate */
+		dl[1] = NULL;
+
+		dom_list = dl;
+		default_domain = dl[0];
 	}
 
 	/***************************
@@ -364,7 +376,7 @@ NTSTATUS idmap_init(void)
 	 */
 	DEBUG(1, ("Initializing idmap domains\n"));
 
-	for (i = 0; dom_list[i]; i++) {
+	for (i=0, num_domains=0; dom_list[i]; i++) {
 	       	const char *parm_backend;
 		char *config_option;
 
@@ -372,12 +384,13 @@ NTSTATUS idmap_init(void)
 		if (strequal(dom_list[i], "BUILTIN")
 		     || strequal(dom_list[i], get_global_sam_name()))
 		{
-			DEBUG(0,("idmap_init: Ignoring invalid domain %s\n",
+			DEBUG(0,("idmap_init: Ignoring domain %s\n",
 				 dom_list[i]));
 			continue;
 		}
 
-		if (strequal(dom_list[i], lp_workgroup())) {
+		if ((dom_list[i] != default_domain) &&
+		    strequal(dom_list[i], lp_workgroup())) {
 			pri_dom_is_in_list = True;
 		}
 		/* init domain */
@@ -398,10 +411,10 @@ NTSTATUS idmap_init(void)
 						   "default", False);
 
 		if (dom->default_domain ||
-		    strequal(dom_list[i], idmap_default_domain[0])) {
+		    (default_domain && strequal(dom_list[i], default_domain))) {
 
 			/* make sure this is set even when we match
-			 * idmap_default_domain[0] */
+			 * default_domain */
 			dom->default_domain = True;
 
 			if (default_already_defined) {
@@ -422,9 +435,10 @@ NTSTATUS idmap_init(void)
 		if (compat) {
 			parm_backend = talloc_strdup(idmap_ctx, compat_backend);
 		} else {
-			char *backend = lp_parm_const_string(-1, config_option,
-							     "backend", "tdb");
-			parm_backend = talloc_strdup(idmap_ctx,	backend);
+			parm_backend = talloc_strdup(idmap_ctx,
+						     lp_parm_const_string(
+							-1, config_option,
+							"backend", "tdb"));
 		}
 		IDMAP_CHECK_ALLOC(parm_backend);
 
@@ -478,13 +492,17 @@ NTSTATUS idmap_init(void)
 			ret = NT_STATUS_NO_MEMORY;
 			goto done;
 		}
-		idmap_domains[i] = dom;
+		idmap_domains[num_domains] = dom;
 
 		/* save default domain position for future uses */
 		if (dom->default_domain) {
-			def_dom_num = i;
+			def_dom_num = num_domains;
 		}
 
+		/* Bump counter to next available slot */
+
+		num_domains++;
+
 		DEBUG(10, ("Domain %s - Backend %s - %sdefault - %sreadonly\n",
 				dom->name, parm_backend,
 				dom->default_domain?"":"not ",
@@ -493,9 +511,6 @@ NTSTATUS idmap_init(void)
 		talloc_free(config_option);
 	}
 
-	/* save the number of domains we have */
-	num_domains = i;
-
 	/* automatically add idmap_nss backend if needed */
 	if ((lp_server_role() == ROLE_DOMAIN_MEMBER) &&
 	    ( ! pri_dom_is_in_list) &&
