@@ -303,6 +303,9 @@ static void process_request(struct winbindd_cli_state *state)
 	if (state->mem_ctx == NULL)
 		return;
 
+	/* Remember who asked us. */
+	state->pid = state->request.pid;
+
 	/* Process command */
 
 	for (table = dispatch_table; table->fn; table++) {
@@ -537,8 +540,8 @@ static void request_len_recv(void *private_data, BOOL success)
 	}
 
 	if (*(uint32 *)(&state->request) != sizeof(state->request)) {
-		DEBUG(0,("request_len_recv: Invalid request size received: %d\n",
-			 *(uint32 *)(&state->request)));
+		DEBUG(0,("request_len_recv: Invalid request size received: %d (expected %d)\n",
+			 *(uint32 *)(&state->request), sizeof(state->request)));
 		state->finished = True;
 		return;
 	}
