@@ -249,9 +249,7 @@ static void check_domain_online_handler(struct event_context *ctx,
 	DEBUG(10,("check_domain_online_handler: called for domain %s\n",
 		domain->name ));
 
-	if (domain->check_online_event) {
-		TALLOC_FREE(domain->check_online_event);
-	}
+	TALLOC_FREE(domain->check_online_event);
 
 	/* Are we still in "startup" mode ? */
 
@@ -303,9 +301,7 @@ void set_domain_offline(struct winbindd_domain *domain)
 	DEBUG(10,("set_domain_offline: called for domain %s\n",
 		domain->name ));
 
-	if (domain->check_online_event) {
-		TALLOC_FREE(domain->check_online_event);
-	}
+	TALLOC_FREE(domain->check_online_event);
 
 	if (domain->internal) {
 		DEBUG(3,("set_domain_offline: domain %s is internal - logic error.\n",
@@ -400,9 +396,7 @@ static void set_domain_online(struct winbindd_domain *domain)
 
 	/* Ensure we have no online timeout checks. */
 	domain->check_online_timeout = 0;
-	if (domain->check_online_event) {
-		TALLOC_FREE(domain->check_online_event);
-	}
+	TALLOC_FREE(domain->check_online_event);
 
 	/* Ensure we ignore any pending child messages. */
 	message_deregister(MSG_WINBIND_TRY_TO_GO_ONLINE);
@@ -557,8 +551,8 @@ static BOOL get_dc_name_via_netlogon(const struct winbindd_domain *domain,
 
 	orig_timeout = cli_set_timeout(netlogon_pipe->cli, 35000);
 	
-	werr = rpccli_netlogon_getdcname(netlogon_pipe, mem_ctx, our_domain->dcname,
-					   domain->name, tmp);
+	werr = rpccli_netlogon_getanydcname(netlogon_pipe, mem_ctx, our_domain->dcname,
+					    domain->name, tmp);
 
 	/* And restore our original timeout. */
 	cli_set_timeout(netlogon_pipe->cli, orig_timeout);
@@ -566,12 +560,12 @@ static BOOL get_dc_name_via_netlogon(const struct winbindd_domain *domain,
 	talloc_destroy(mem_ctx);
 
 	if (!W_ERROR_IS_OK(werr)) {
-		DEBUG(10, ("rpccli_netlogon_getdcname failed: %s\n",
+		DEBUG(10, ("rpccli_netlogon_getanydcname failed: %s\n",
 			   dos_errstr(werr)));
 		return False;
 	}
 
-	/* cli_netlogon_getdcname gives us a name with \\ */
+	/* cli_netlogon_getanydcname gives us a name with \\ */
 	p = tmp;
 	if (*p == '\\') {
 		p+=1;
@@ -582,7 +576,7 @@ static BOOL get_dc_name_via_netlogon(const struct winbindd_domain *domain,
 
 	fstrcpy(dcname, p);
 
-	DEBUG(10, ("rpccli_netlogon_getdcname returned %s\n", dcname));
+	DEBUG(10, ("rpccli_netlogon_getanydcname returned %s\n", dcname));
 
 	if (!resolve_name(dcname, dc_ip, 0x20)) {
 		return False;
@@ -1038,19 +1032,27 @@ static BOOL dcip_to_name(const struct winbindd_domain *domain, struct in_addr ip
 
 			DEBUG(10,("dcip_to_name: flags = 0x%x\n", (unsigned int)ads->config.flags));
 
-			if (domain->primary && (ads->config.flags & ADS_KDC) && ads_closest_dc(ads)) {
-				char *sitename = sitename_fetch(ads->config.realm);
+			if (domain->primary && (ads->config.flags & ADS_KDC)) {
+				if (ads_closest_dc(ads)) {
+					char *sitename = sitename_fetch(ads->config.realm);
 
-				/* We're going to use this KDC for this realm/domain.
-				   If we are using sites, then force the krb5 libs
-				   to use this KDC. */
+					/* We're going to use this KDC for this realm/domain.
+					   If we are using sites, then force the krb5 libs
+					   to use this KDC. */
 
-				create_local_private_krb5_conf_for_domain(domain->alt_name,
-								domain->name,
-								sitename,
-								ip);
+					create_local_private_krb5_conf_for_domain(domain->alt_name,
+									domain->name,
+									sitename,
+									ip);
 
-				SAFE_FREE(sitename);
+					SAFE_FREE(sitename);
+				} else {
+					/* use an off site KDC */
+					create_local_private_krb5_conf_for_domain(domain->alt_name,
+									domain->name,
+									NULL,
+									ip);
+				}
 				/* Ensure we contact this DC also. */
 				saf_store( domain->name, name);
 				saf_store( domain->alt_name, name);
@@ -1556,6 +1558,16 @@ static void set_dc_type_and_flags( struct winbindd_domain *domain )
 		DEBUG(5, ("set_dc_type_and_flags: rpccli_ds_getprimarydominfo "
 			  "on domain %s failed: (%s)\n",
 			  domain->name, nt_errstr(result)));
+
+		/* older samba3 DCs will return DCERPC_FAULT_OP_RNG_ERROR for
+		 * every opcode on the LSARPC_DS pipe, continue with
+		 * no_lsarpc_ds mode here as well to get domain->initialized
+		 * set - gd */
+
+		if (NT_STATUS_V(result) == DCERPC_FAULT_OP_RNG_ERROR) {
+			goto no_lsarpc_ds;
+		}
+
 		return;
 	}
 	
