@@ -56,6 +56,9 @@
  *
  *  Modified to call CUPS support if printcap name is set to "cups"
  *  in smb.conf.
+ *
+ *  Modified to call iPrint support if printcap name is set to "iprint"
+ *  in smb.conf.
  */
 
 #include "includes.h"
@@ -129,7 +132,14 @@ void pcap_cache_reload(void)
 	}
 #endif
 
-#ifdef SYSV
+#ifdef HAVE_IPRINT
+	if (strequal(pcap_name, "iprint")) {
+		pcap_reloaded = iprint_cache_reload();
+		goto done;
+	}
+#endif
+
+#if defined(SYSV) || defined(HPUX)
 	if (strequal(pcap_name, "lpstat")) {
 		pcap_reloaded = sysv_cache_reload();
 		goto done;
