@@ -181,7 +181,6 @@ NTSTATUS rpccli_lsa_lookup_sids(struct rpc_pipe_client *cli,
 	LSA_Q_LOOKUP_SIDS q;
 	LSA_R_LOOKUP_SIDS r;
 	DOM_R_REF ref;
-	LSA_TRANS_NAME_ENUM t_names;
 	NTSTATUS result = NT_STATUS_OK;
 	int i;
 
@@ -191,10 +190,8 @@ NTSTATUS rpccli_lsa_lookup_sids(struct rpc_pipe_client *cli,
 	init_q_lookup_sids(mem_ctx, &q, pol, num_sids, sids, 1);
 
 	ZERO_STRUCT(ref);
-	ZERO_STRUCT(t_names);
 
 	r.dom_ref = &ref;
-	r.names = &t_names;
 
 	CLI_DO_RPC( cli, mem_ctx, PI_LSARPC, LSA_LOOKUPSIDS,
 			q, r,
@@ -245,7 +242,7 @@ NTSTATUS rpccli_lsa_lookup_sids(struct rpc_pipe_client *cli,
 		
 	for (i = 0; i < num_sids; i++) {
 		fstring name, dom_name;
-		uint32 dom_idx = t_names.name[i].domain_idx;
+		uint32 dom_idx = r.names.name[i].domain_idx;
 
 		/* Translate optimised name through domain index array */
 
@@ -254,11 +251,11 @@ NTSTATUS rpccli_lsa_lookup_sids(struct rpc_pipe_client *cli,
 			rpcstr_pull_unistr2_fstring(
                                 dom_name, &ref.ref_dom[dom_idx].uni_dom_name);
 			rpcstr_pull_unistr2_fstring(
-                                name, &t_names.uni_name[i]);
+                                name, &r.names.uni_name[i]);
 
 			(*names)[i] = talloc_strdup(mem_ctx, name);
 			(*domains)[i] = talloc_strdup(mem_ctx, dom_name);
-			(*types)[i] = t_names.name[i].sid_name_use;
+			(*types)[i] = r.names.name[i].sid_name_use;
 			
 			if (((*names)[i] == NULL) || ((*domains)[i] == NULL)) {
 				DEBUG(0, ("cli_lsa_lookup_sids(): out of memory\n"));
