@@ -380,12 +380,10 @@ static void init_reply_lookup_names4(LSA_R_LOOKUP_NAMES4 *r_l,
 
 static void init_reply_lookup_sids2(LSA_R_LOOKUP_SIDS2 *r_l,
 				DOM_R_REF *ref,
-				LSA_TRANS_NAME_ENUM2 *names,
 				uint32 mapped_count)
 {
 	r_l->ptr_dom_ref  = ref ? 1 : 0;
 	r_l->dom_ref      = ref;
-	r_l->names        = names;
 	r_l->mapped_count = mapped_count;
 }
 
@@ -395,12 +393,10 @@ static void init_reply_lookup_sids2(LSA_R_LOOKUP_SIDS2 *r_l,
 
 static void init_reply_lookup_sids3(LSA_R_LOOKUP_SIDS3 *r_l,
 				DOM_R_REF *ref,
-				LSA_TRANS_NAME_ENUM2 *names,
 				uint32 mapped_count)
 {
 	r_l->ptr_dom_ref  = ref ? 1 : 0;
 	r_l->dom_ref      = ref;
-	r_l->names        = names;
 	r_l->mapped_count = mapped_count;
 }
 
@@ -414,11 +410,7 @@ static NTSTATUS init_reply_lookup_sids(TALLOC_CTX *mem_ctx,
 				LSA_TRANS_NAME_ENUM2 *names,
 				uint32 mapped_count)
 {
-	LSA_TRANS_NAME_ENUM *oldnames = TALLOC_ZERO_P(mem_ctx, LSA_TRANS_NAME_ENUM);
-
-	if (!oldnames) {
-		return NT_STATUS_NO_MEMORY;
-	}
+	LSA_TRANS_NAME_ENUM *oldnames = &r_l->names;
 
 	oldnames->num_entries = names->num_entries;
 	oldnames->ptr_trans_names = names->ptr_trans_names;
@@ -428,7 +420,7 @@ static NTSTATUS init_reply_lookup_sids(TALLOC_CTX *mem_ctx,
 	if (names->num_entries) {
 		int i;
 
-		oldnames->name = TALLOC_ARRAY(oldnames, LSA_TRANS_NAME, names->num_entries);
+		oldnames->name = TALLOC_ARRAY(mem_ctx, LSA_TRANS_NAME, names->num_entries);
 
 		if (!oldnames->name) {
 			return NT_STATUS_NO_MEMORY;
@@ -442,7 +434,6 @@ static NTSTATUS init_reply_lookup_sids(TALLOC_CTX *mem_ctx,
 
 	r_l->ptr_dom_ref  = ref ? 1 : 0;
 	r_l->dom_ref      = ref;
-	r_l->names        = oldnames;
 	r_l->mapped_count = mapped_count;
 	return NT_STATUS_OK;
 }
@@ -811,13 +802,12 @@ static NTSTATUS _lsa_lookup_sids_internal(pipes_struct *p,
 				int num_sids,				/* input */
 				const DOM_SID2 *sid,			/* input */
 				DOM_R_REF **pp_ref,			/* output */
-				LSA_TRANS_NAME_ENUM2 **pp_names,	/* output */
+				LSA_TRANS_NAME_ENUM2 *names,		/* input/output */
 				uint32 *pp_mapped_count)
 {
 	NTSTATUS status;
 	int i;
 	const DOM_SID **sids = NULL;
-	LSA_TRANS_NAME_ENUM2 *names = NULL;
 	DOM_R_REF *ref = NULL;
 	uint32 mapped_count = 0;
 	struct lsa_dom_info *dom_infos = NULL;
@@ -825,17 +815,16 @@ static NTSTATUS _lsa_lookup_sids_internal(pipes_struct *p,
 
 	*pp_mapped_count = 0;
 	*pp_ref = NULL;
-	*pp_names = NULL;
+	ZERO_STRUCTP(names);
 
 	if (num_sids == 0) {
 		return NT_STATUS_OK;
 	}
 
-	names = TALLOC_ZERO_P(p->mem_ctx, LSA_TRANS_NAME_ENUM2);
 	sids = TALLOC_ARRAY(p->mem_ctx, const DOM_SID *, num_sids);
 	ref = TALLOC_ZERO_P(p->mem_ctx, DOM_R_REF);
 
-	if (sids == NULL || names == NULL || ref == NULL) {
+	if (sids == NULL || ref == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
 
@@ -850,8 +839,8 @@ static NTSTATUS _lsa_lookup_sids_internal(pipes_struct *p,
 		return status;
 	}
 
-	names->name = TALLOC_ARRAY(names, LSA_TRANS_NAME2, num_sids);
-	names->uni_name = TALLOC_ARRAY(names, UNISTR2, num_sids);
+	names->name = TALLOC_ARRAY(p->mem_ctx, LSA_TRANS_NAME2, num_sids);
+	names->uni_name = TALLOC_ARRAY(p->mem_ctx, UNISTR2, num_sids);
 	if ((names->name == NULL) || (names->uni_name == NULL)) {
 		return NT_STATUS_NO_MEMORY;
 	}
@@ -903,7 +892,6 @@ static NTSTATUS _lsa_lookup_sids_internal(pipes_struct *p,
 
 	*pp_mapped_count = mapped_count;
 	*pp_ref = ref;
-	*pp_names = names;
 
 	return status;
 }
@@ -920,7 +908,7 @@ NTSTATUS _lsa_lookup_sids(pipes_struct *p,
 	int num_sids = q_u->sids.num_entries;
 	uint32 mapped_count = 0;
 	DOM_R_REF *ref = NULL;
-	LSA_TRANS_NAME_ENUM2 *names = NULL;
+	LSA_TRANS_NAME_ENUM2 names;
 	NTSTATUS status;
 
 	if ((q_u->level < 1) || (q_u->level > 6)) {
@@ -952,7 +940,7 @@ NTSTATUS _lsa_lookup_sids(pipes_struct *p,
 
 	/* Convert from LSA_TRANS_NAME_ENUM2 to LSA_TRANS_NAME_ENUM */
 
-	status = init_reply_lookup_sids(p->mem_ctx, r_u, ref, names, mapped_count);
+	status = init_reply_lookup_sids(p->mem_ctx, r_u, ref, &names, mapped_count);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -971,7 +959,6 @@ NTSTATUS _lsa_lookup_sids2(pipes_struct *p,
 	int num_sids = q_u->sids.num_entries;
 	uint32 mapped_count = 0;
 	DOM_R_REF *ref = NULL;
-	LSA_TRANS_NAME_ENUM2 *names = NULL;
 
 	if ((q_u->level < 1) || (q_u->level > 6)) {
 		return NT_STATUS_INVALID_PARAMETER;
@@ -997,10 +984,10 @@ NTSTATUS _lsa_lookup_sids2(pipes_struct *p,
 						num_sids, 
 						q_u->sids.sid,
 						&ref,
-						&names,
+						&r_u->names,
 						&mapped_count);
 
-	init_reply_lookup_sids2(r_u, ref, names, mapped_count);
+	init_reply_lookup_sids2(r_u, ref, mapped_count);
 	return r_u->status;
 }
 
@@ -1015,7 +1002,6 @@ NTSTATUS _lsa_lookup_sids3(pipes_struct *p,
 	int num_sids = q_u->sids.num_entries;
 	uint32 mapped_count = 0;
 	DOM_R_REF *ref = NULL;
-	LSA_TRANS_NAME_ENUM2 *names = NULL;
 
 	if ((q_u->level < 1) || (q_u->level > 6)) {
 		return NT_STATUS_INVALID_PARAMETER;
@@ -1039,13 +1025,38 @@ NTSTATUS _lsa_lookup_sids3(pipes_struct *p,
 						num_sids, 
 						q_u->sids.sid,
 						&ref,
-						&names,
+						&r_u->names,
 						&mapped_count);
 
-	init_reply_lookup_sids3(r_u, ref, names, mapped_count);
+	init_reply_lookup_sids3(r_u, ref, mapped_count);
 	return r_u->status;
 }
 
+static int lsa_lookup_level_to_flags(uint16 level)
+{
+	int flags;
+
+	switch (level) {
+		case 1:
+			flags = LOOKUP_NAME_ALL;
+			break;
+		case 2:
+			flags = LOOKUP_NAME_DOMAIN|LOOKUP_NAME_REMOTE|LOOKUP_NAME_ISOLATED;
+			break;
+		case 3:
+			flags = LOOKUP_NAME_DOMAIN|LOOKUP_NAME_ISOLATED;
+			break;
+		case 4:
+		case 5:
+		case 6:
+		default:
+			flags = LOOKUP_NAME_NONE;
+			break;
+	}
+
+	return flags;
+}
+
 /***************************************************************************
 lsa_reply_lookup_names
  ***************************************************************************/
@@ -1065,10 +1076,7 @@ NTSTATUS _lsa_lookup_names(pipes_struct *p,LSA_Q_LOOKUP_NAMES *q_u, LSA_R_LOOKUP
 		DEBUG(5,("_lsa_lookup_names: truncating name lookup list to %d\n", num_entries));
 	}
 		
-	/* Probably the lookup_level is some sort of bitmask. */
-	if (q_u->lookup_level == 1) {
-		flags = LOOKUP_NAME_ALL;
-	}
+	flags = lsa_lookup_level_to_flags(q_u->lookup_level);
 
 	ref = TALLOC_ZERO_P(p->mem_ctx, DOM_R_REF);
 	if (!ref) {
@@ -1134,11 +1142,8 @@ NTSTATUS _lsa_lookup_names2(pipes_struct *p, LSA_Q_LOOKUP_NAMES2 *q_u, LSA_R_LOO
 		num_entries = MAX_LOOKUP_SIDS;
 		DEBUG(5,("_lsa_lookup_names2: truncating name lookup list to %d\n", num_entries));
 	}
-		
-	/* Probably the lookup_level is some sort of bitmask. */
-	if (q_u->lookup_level == 1) {
-		flags = LOOKUP_NAME_ALL;
-	}
+
+	flags = lsa_lookup_level_to_flags(q_u->lookup_level);
 
 	ref = TALLOC_ZERO_P(p->mem_ctx, DOM_R_REF);
 	if (ref == NULL) {
