@@ -414,7 +414,7 @@ static int do_ntcreate_pipe_open(connection_struct *conn,
 	p += 4;
 
 	if (flags & EXTENDED_RESPONSE_REQUIRED) {
-		p += 26;
+		p += 25;
 		SIVAL(p,0,FILE_GENERIC_ALL);
 		/* 
 		 * For pipes W2K3 seems to return
@@ -543,7 +543,6 @@ int reply_ntcreate_and_X(connection_struct *conn,
 		 */
 		pstring rel_fname;
 		files_struct *dir_fsp = file_fsp(inbuf,smb_ntcreate_RootDirectoryFid);
-		size_t dir_name_len;
 
 		if(!dir_fsp) {
 			END_PROFILE(SMBntcreateX);
@@ -583,15 +582,18 @@ int reply_ntcreate_and_X(connection_struct *conn,
 		 */
 
 		pstrcpy( fname, dir_fsp->fsp_name );
-		dir_name_len = strlen(fname);
 
-		/*
-		 * Ensure it ends in a '\'.
-		 */
+		if (ISDOT(fname)) {
+			fname[0] = '\0';
+		} else {
+			size_t dir_name_len = strlen(fname);
+			/*
+			 * Ensure it ends in a '\'.
+			 */
 
-		if((fname[dir_name_len-1] != '\\') && (fname[dir_name_len-1] != '/')) {
-			pstrcat(fname, "/");
-			dir_name_len++;
+			if((fname[dir_name_len-1] != '\\') && (fname[dir_name_len-1] != '/')) {
+				pstrcat(fname, "/");
+			}
 		}
 
 		srvstr_get_path(inbuf, rel_fname, smb_buf(inbuf), sizeof(rel_fname), 0, STR_TERMINATE, &status);
@@ -716,18 +718,6 @@ int reply_ntcreate_and_X(connection_struct *conn,
 					create_options,
 					new_file_attributes,
 					&info, &fsp);
-
-		restore_case_semantics(conn, file_attributes);
-
-		if(!NT_STATUS_IS_OK(status)) {
-			if (!use_nt_status() && NT_STATUS_EQUAL(
-				    status, NT_STATUS_OBJECT_NAME_COLLISION)) {
-				status = NT_STATUS_DOS(ERRDOS, ERRfilexists);
-			}
-			END_PROFILE(SMBntcreateX);
-			return ERROR_NT(status);
-		}
-
 	} else {
 
 		/*
@@ -756,7 +746,7 @@ int reply_ntcreate_and_X(connection_struct *conn,
 					oplock_request,
 					&info, &fsp);
 
-		if (!NT_STATUS_IS_OK(status)) { 
+		if (!NT_STATUS_IS_OK(status)) {
 			/* We cheat here. There are two cases we
 			 * care about. One is a directory rename,
 			 * where the NT client will attempt to
@@ -788,7 +778,7 @@ int reply_ntcreate_and_X(connection_struct *conn,
 					END_PROFILE(SMBntcreateX);
 					return ERROR_FORCE_NT(NT_STATUS_FILE_IS_A_DIRECTORY);
 				}
-	
+
 				oplock_request = 0;
 				status = open_directory(conn, fname, &sbuf,
 							access_mask,
@@ -798,29 +788,23 @@ int reply_ntcreate_and_X(connection_struct *conn,
 							new_file_attributes,
 							&info, &fsp);
 
-				if(!NT_STATUS_IS_OK(status)) {
-					restore_case_semantics(conn, file_attributes);
-					if (!use_nt_status() && NT_STATUS_EQUAL(
-						    status, NT_STATUS_OBJECT_NAME_COLLISION)) {
-						status = NT_STATUS_DOS(ERRDOS, ERRfilexists);
-					}
-					END_PROFILE(SMBntcreateX);
-					return ERROR_NT(status);
-				}
-			} else {
-				restore_case_semantics(conn, file_attributes);
-				END_PROFILE(SMBntcreateX);
-				if (open_was_deferred(SVAL(inbuf,smb_mid))) {
-					/* We have re-scheduled this call. */
-					return -1;
-				}
-				return ERROR_NT(status);
 			}
-		} 
+		}
 	}
-		
+
 	restore_case_semantics(conn, file_attributes);
 
+	if(!NT_STATUS_IS_OK(status)) {
+		END_PROFILE(SMBntcreateX);
+
+		if (open_was_deferred(SVAL(inbuf,smb_mid))) {
+			/* We have re-scheduled this call. */
+			return -1;
+		}
+
+		return ERROR_OPEN(status);
+	}
+
 	file_len = sbuf.st_size;
 	fattr = dos_mode(conn,fname,&sbuf);
 	if(fattr == 0) {
@@ -944,7 +928,7 @@ int reply_ntcreate_and_X(connection_struct *conn,
 
 	if (flags & EXTENDED_RESPONSE_REQUIRED) {
 		uint32 perms = 0;
-		p += 26;
+		p += 25;
 		if (fsp->is_directory || can_write_to_file(conn, fname, &sbuf)) {
 			perms = FILE_GENERIC_ALL;
 		} else {
@@ -1029,7 +1013,7 @@ static int do_nt_transact_create_pipe( connection_struct *conn, char *inbuf, cha
 	p += 4;
 	
 	if (flags & EXTENDED_RESPONSE_REQUIRED) {
-		p += 26;
+		p += 25;
 		SIVAL(p,0,FILE_GENERIC_ALL);
 		/* 
 		 * For pipes W2K3 seems to return
@@ -1262,7 +1246,6 @@ static int call_nt_transact_create(connection_struct *conn, char *inbuf, char *o
 		 * This filename is relative to a directory fid.
 		 */
 		files_struct *dir_fsp = file_fsp(params,4);
-		size_t dir_name_len;
 
 		if(!dir_fsp) {
 			return ERROR_DOS(ERRDOS,ERRbadfid);
@@ -1290,15 +1273,18 @@ static int call_nt_transact_create(connection_struct *conn, char *inbuf, char *o
 		 */
 
 		pstrcpy( fname, dir_fsp->fsp_name );
-		dir_name_len = strlen(fname);
 
-		/*
-		 * Ensure it ends in a '\'.
-		 */
+                if (ISDOT(fname)) {
+			fname[0] = '\0';
+		} else {
+			size_t dir_name_len = strlen(fname);
+			/*
+			 * Ensure it ends in a '\'.
+			 */
 
-		if((fname[dir_name_len-1] != '\\') && (fname[dir_name_len-1] != '/')) {
-			pstrcat(fname, "/");
-			dir_name_len++;
+			if((fname[dir_name_len-1] != '\\') && (fname[dir_name_len-1] != '/')) {
+				pstrcat(fname, "/");
+			}
 		}
 
 		{
@@ -1416,11 +1402,6 @@ static int call_nt_transact_create(connection_struct *conn, char *inbuf, char *o
 					create_options,
 					new_file_attributes,
 					&info, &fsp);
-		if(!NT_STATUS_IS_OK(status)) {
-			restore_case_semantics(conn, file_attributes);
-			return ERROR_NT(status);
-		}
-
 	} else {
 
 		/*
@@ -1436,7 +1417,7 @@ static int call_nt_transact_create(connection_struct *conn, char *inbuf, char *o
 					oplock_request,
 					&info, &fsp);
 
-		if (!NT_STATUS_IS_OK(status)) { 
+		if (!NT_STATUS_IS_OK(status)) {
 			if (NT_STATUS_EQUAL(status,
 					    NT_STATUS_FILE_IS_A_DIRECTORY)) {
 
@@ -1448,7 +1429,7 @@ static int call_nt_transact_create(connection_struct *conn, char *inbuf, char *o
 					restore_case_semantics(conn, file_attributes);
 					return ERROR_FORCE_NT(NT_STATUS_FILE_IS_A_DIRECTORY);
 				}
-	
+
 				oplock_request = 0;
 				status = open_directory(conn, fname, &sbuf,
 							access_mask,
@@ -1457,26 +1438,26 @@ static int call_nt_transact_create(connection_struct *conn, char *inbuf, char *o
 							create_options,
 							new_file_attributes,
 							&info, &fsp);
-				if(!NT_STATUS_IS_OK(status)) {
-					restore_case_semantics(conn, file_attributes);
-					return ERROR_NT(status);
-				}
-			} else {
-				restore_case_semantics(conn, file_attributes);
-				if (open_was_deferred(SVAL(inbuf,smb_mid))) {
-					/* We have re-scheduled this call. */
-					return -1;
-				}
-				return ERROR_NT(status);
 			}
-		} 
+		}
+	}
+
+	restore_case_semantics(conn, file_attributes);
+	if(!NT_STATUS_IS_OK(status)) {
+
+		if (open_was_deferred(SVAL(inbuf,smb_mid))) {
+			/* We have re-scheduled this call. */
+			return -1;
+		}
+
+		return ERROR_OPEN(status);
 	}
 
 	/*
 	 * According to the MS documentation, the only time the security
 	 * descriptor is applied to the opened file is iff we *created* the
 	 * file; an existing file stays the same.
-	 * 
+	 *
 	 * Also, it seems (from observation) that you can open the file with
 	 * any access mask but you can still write the sd. We need to override
 	 * the granted access before we call set_sd
@@ -1625,7 +1606,7 @@ static int call_nt_transact_create(connection_struct *conn, char *inbuf, char *o
 
 	if (flags & EXTENDED_RESPONSE_REQUIRED) {
 		uint32 perms = 0;
-		p += 26;
+		p += 25;
 		if (fsp->is_directory || can_write_to_file(conn, fname, &sbuf)) {
 			perms = FILE_GENERIC_ALL;
 		} else {
@@ -1979,8 +1960,7 @@ static int call_nt_transact_notify_change(connection_struct *conn, char *inbuf,
 		 * here.
 		 */
 
-		change_notify_reply(inbuf, max_param_count,
-				    fsp->notify);
+		change_notify_reply(inbuf, max_param_count, fsp->notify);
 
 		/*
 		 * change_notify_reply() above has independently sent its
@@ -1994,7 +1974,7 @@ static int call_nt_transact_notify_change(connection_struct *conn, char *inbuf,
 	 */
 
 	status = change_notify_add_request(inbuf, max_param_count, filter,
-					   recursive, fsp);
+			recursive, fsp);
 	if (!NT_STATUS_IS_OK(status)) {
 		return ERROR_NT(status);
 	}
