@@ -28,6 +28,8 @@
  * + and & may be combined
  */
 
+extern userdom_struct current_user_info;
+
 static BOOL do_group_checks(const char **name, const char **pattern)
 {
 	if ((*name)[0] == '@') {
@@ -71,10 +73,11 @@ static BOOL token_contains_name(TALLOC_CTX *mem_ctx,
 {
 	const char *prefix;
 	DOM_SID sid;
-	enum SID_NAME_USE type;
+	enum lsa_SidType type;
 
 	if (username != NULL) {
-		name = talloc_sub_basic(mem_ctx, username, name);
+		name = talloc_sub_basic(mem_ctx, username,
+					current_user_info.domain, name);
 	}
 	if (sharename != NULL) {
 		name = talloc_string_sub(mem_ctx, name, "%S", sharename);
